@@ -128,10 +128,15 @@ public:
     return m_stylesheetStack.top();
   }
 
+  // use to create the final table name in numbers document
+  virtual boost::optional<std::string> getWorkSpaceName() const
+  {
+    return boost::none;
+  }
   IWORKOutputManager &getOutputManager();
 
 public:
-  virtual std::shared_ptr<IWORKTable> createTable(const IWORKTableNameMapPtr_t &tableNameMap, const IWORKLanguageManager &langManager) const;
+  virtual std::shared_ptr<IWORKTable> createTable(const IWORKTableNameMapPtr_t &tableNameMap, IWORKFormatNameMap &formatNameMap, const IWORKLanguageManager &langManager) const;
   virtual std::shared_ptr<IWORKText> createText(const IWORKLanguageManager &langManager, bool discardEmptyContent = false, bool allowListInsertion=true) const;
 
 protected:
