@@ -18,7 +18,7 @@
 #include <boost/fusion/adapted/std_pair.hpp>
 #include <boost/fusion/include/adapt_struct.hpp>
 #include <boost/spirit/include/qi.hpp>
-#include <boost/spirit/include/phoenix.hpp>
+#include <boost/phoenix.hpp>
 #include <boost/variant/recursive_variant.hpp>
 
 namespace libetonyek
@@ -32,31 +32,49 @@ using boost::variant;
 using std::string;
 using std::vector;
 
-struct Coord
+std::ostream &operator<<(std::ostream &s, IWORKFormula::Address const &ad)
 {
-  Coord()
-    : m_coord(0)
-    , m_absolute(false)
+  if (ad.m_table) s << "[" << get(ad.m_table) << "]";
+  if (ad.m_column)
   {
+    if (get(ad.m_column).m_absolute) s << "$";
+    s << "C" << get(ad.m_column).m_coord;
   }
-  unsigned m_coord;
-  bool m_absolute;
-};
+  if (ad.m_row)
+  {
+    if (get(ad.m_row).m_absolute) s << "$";
+    s << "R" << get(ad.m_row).m_coord;
+  }
+  return s;
+}
 
-struct Address
+std::ostream &operator<<(std::ostream &s, IWORKFormula::Token const &dt)
 {
-  Address()
-    : m_column()
-    , m_row()
-    , m_table()
+  switch (dt.m_type)
   {
+  case IWORKFormula::Token::Cell:
+    s << dt.m_address;
+    break;
+  case IWORKFormula::Token::Double:
+    s << dt.m_value;
+    break;
+  case IWORKFormula::Token::Operator:
+  case IWORKFormula::Token::Function:
+    s << dt.m_string;
+    break;
+  case IWORKFormula::Token::String:
+    s << "\"" << dt.m_string << "\"";
+    break;
+#if !defined(__clang__)
+  default:
+    ETONYEK_DEBUG_MSG(("IWORKFormula::Token::operator<<: unexpected token\n"));
+    break;
+#endif
   }
-  optional<Coord> m_column;
-  optional<Coord> m_row;
-  optional<string> m_table;
-};
+  return s;
+}
 
-typedef std::pair<Address, Address> AddressRange;
+typedef std::pair<IWORKFormula::Address, IWORKFormula::Address> AddressRange;
 
 struct TrueOrFalseFunc
 {
@@ -73,7 +91,7 @@ struct InfixOp;
 struct PostfixOp;
 struct PExpr;
 
-typedef variant<double, string, TrueOrFalseFunc, Address, AddressRange, recursive_wrapper<PrefixOp>, recursive_wrapper<InfixOp>, recursive_wrapper<PostfixOp>, recursive_wrapper<Function>, recursive_wrapper<PExpr> > Expression;
+typedef variant<double, string, TrueOrFalseFunc, IWORKFormula::Address, AddressRange, recursive_wrapper<PrefixOp>, recursive_wrapper<InfixOp>, recursive_wrapper<PostfixOp>, recursive_wrapper<Function>, recursive_wrapper<PExpr> > Expression;
 
 struct PrefixOp
 {
@@ -134,9 +152,9 @@ struct PExpr
 }
 
 BOOST_FUSION_ADAPT_STRUCT(
-  libetonyek::Coord,
+  libetonyek::IWORKFormula::Coord,
   (bool, m_absolute)
-  (unsigned, m_coord)
+  (int, m_coord)
 )
 
 BOOST_FUSION_ADAPT_STRUCT(
@@ -145,10 +163,10 @@ BOOST_FUSION_ADAPT_STRUCT(
 )
 
 BOOST_FUSION_ADAPT_STRUCT(
-  libetonyek::Address,
+  libetonyek::IWORKFormula::Address,
   (optional<std::string>, m_table)
-  (optional<libetonyek::Coord>, m_column)
-  (optional<libetonyek::Coord>, m_row)
+  (optional<libetonyek::IWORKFormula::Coord>, m_column)
+  (optional<libetonyek::IWORKFormula::Coord>, m_row)
 )
 
 BOOST_FUSION_ADAPT_STRUCT(
@@ -198,7 +216,6 @@ unsigned parseRowName(const vector<char> &columnName)
 
   for (char it : columnName)
     columnNumber = 26 * columnNumber + unsigned(toupper(it) - 'A') + 1;
-
   return columnNumber;
 }
 
@@ -220,6 +237,7 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
     , pExpr()
     , address()
     , addressSpecialColumn()
+    , addressSpecialNoTableRow()
     , addressSpecialRow()
     , range()
     , columnName()
@@ -235,7 +253,6 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
     , postfixOp()
     , prefixLit()
     , postfixLit()
-    , rangeSpecial()
     , mappedName()
   {
     using ascii::char_;
@@ -276,33 +293,28 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
       | attr(false) >> columnName
       ;
 
-    columnName = (+alpha)[_val = bind(parseRowName, _1)];
+    columnName = (+alpha)[_val = bind(parseRowName, _1)] >> !lit("_");
 
-    // TODO: improve
-    table %= +(char_ - ":");
+    table %= lit("SFTGlobalID_") >> +char_("0-9a-fA-F");
 
     address %=
-      table >> "::" >> column >> row
+      table >> *lit(' ') >> "::" >> *lit(' ') >> column >> row
       | attr(none) >> column >> row
       ;
 
     addressSpecialColumn %=
-      table >> "::" >> column >> attr(none)
+      table >> *lit(' ') >> "::" >> *lit(' ') >> column >> attr(none)
       | attr(none) >> column >> attr(none)
       ;
 
-    addressSpecialRow %=
-      table >> "::" >> attr(none) >> row
-      | attr(none) >> attr(none) >> row
-      ;
-
-    rangeSpecial %= addressSpecialColumn[_a = _1] >> attr(_a) >> !row;
+    addressSpecialRow %= table >> *lit(' ') >> "::" >> *lit(' ') >> attr(none) >> row;
+    addressSpecialNoTableRow %= attr(none) >> attr(none) >> row;
 
     range %=
       address >> ':' >> address
       | addressSpecialColumn >> ':' >> addressSpecialColumn
       | addressSpecialRow >> ':' >> addressSpecialRow
-      | rangeSpecial
+      | addressSpecialNoTableRow >> ":" >> addressSpecialNoTableRow
       ;
 
     prefixOp %= prefixLit >> term;
@@ -320,6 +332,8 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
       | trueOrFalseFunction
       | range
       | address
+      | addressSpecialColumn
+      | addressSpecialRow
       | number
       | prefixOp
       | pExpr
@@ -350,7 +364,6 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
     term.name("term");
     formula.name("formula");
     pExpr.name("pExpr");
-    rangeSpecial.name("rangeSpecial");
   }
 
   void createFunctionNameMap()
@@ -361,17 +374,16 @@ struct FormulaGrammar : public qi::grammar<Iterator, Expression()>
   qi::rule<Iterator, TrueOrFalseFunc()> trueOrFalseFunction;
   qi::rule<Iterator, Expression()> expression, formula, term;
   qi::rule<Iterator, PExpr()> pExpr;
-  qi::rule<Iterator, Address()> address, addressSpecialColumn, addressSpecialRow;
+  qi::rule<Iterator, IWORKFormula::Address()> address, addressSpecialColumn, addressSpecialNoTableRow, addressSpecialRow;
   qi::rule<Iterator, AddressRange()> range;
   qi::rule<Iterator, unsigned()> columnName;
-  qi::rule<Iterator, Coord()> column, row;
+  qi::rule<Iterator, IWORKFormula::Coord()> column, row;
   qi::rule<Iterator, double()> number;
   qi::rule<Iterator, string()> str, table, infixLit, trueOrFalseFunctionLit;
   qi::rule<Iterator, PrefixOp()> prefixOp;
   qi::rule<Iterator, InfixOp()> infixOp;
   qi::rule<Iterator, PostfixOp()> postfixOp;
   qi::rule<Iterator, char()> prefixLit, postfixLit;
-  qi::rule<Iterator, qi::locals<Address> > rangeSpecial;
 
   qi::symbols<char, string> mappedName;
 };
@@ -406,7 +418,7 @@ struct Printer : public boost::static_visitor<void>
     m_out << val;
   }
 
-  void operator()(const Address &val) const
+  void operator()(const IWORKFormula::Address &val) const
   {
     m_out << '[';
     formatAddress(val);
@@ -422,6 +434,29 @@ struct Printer : public boost::static_visitor<void>
     m_out << ']';
   }
 
+  void operator()(const IWORKFormula::Token &val) const
+  {
+    switch (val.m_type)
+    {
+    case IWORKFormula::Token::Cell:
+      operator()(val.m_address);
+      break;
+    case IWORKFormula::Token::Double:
+      m_out << val.m_value;
+      break;
+    case IWORKFormula::Token::Function:
+    case IWORKFormula::Token::Operator:
+    case IWORKFormula::Token::String:
+      m_out << val.m_string;
+      break;
+#if !defined(__clang__)
+    default:
+      ETONYEK_DEBUG_MSG(("IWORKFormula::Printer::operator(): unexpected token\n"));
+      break;
+#endif
+    }
+  }
+
   void operator()(const TrueOrFalseFunc &val) const
   {
     m_out << val.m_name << "()";
@@ -466,7 +501,7 @@ struct Printer : public boost::static_visitor<void>
   }
 
 private:
-  void formatAddress(const Address &val) const
+  void formatAddress(const IWORKFormula::Address &val) const
   {
     if (val.m_table)
       m_out << get(val.m_table);
@@ -555,7 +590,7 @@ struct Collector : public boost::static_visitor<>
     m_propsVector.append(props2);
   }
 
-  void operator()(const Address &val) const
+  void operator()(const IWORKFormula::Address &val) const
   {
 
     librevenge::RVNGPropertyList props;
@@ -563,11 +598,24 @@ struct Collector : public boost::static_visitor<>
 
     if (val.m_table)
     {
-      const IWORKTableNameMap_t::const_iterator it = m_tableNameMap->find(get(val.m_table));
-      if (m_tableNameMap->end() != it)
-        props.insert("librevenge:sheet-name", (it->second).c_str());
+      std::string tableName("SFTGlobalID_");
+      tableName+=get(val.m_table);
+      if (m_tableNameMap)
+      {
+        auto it = m_tableNameMap->find(tableName);
+        if (m_tableNameMap->end() != it)
+          props.insert("librevenge:sheet-name", (it->second).c_str());
+        else
+        {
+          ETONYEK_DEBUG_MSG(("IWORKFormula::operator()(IWORKFormula::Address): can not find the table name: %s\n", tableName.c_str()));
+          props.insert("librevenge:sheet-name", tableName.c_str());
+        }
+      }
       else
-        props.insert("librevenge:sheet-name", get(val.m_table).c_str());
+      {
+        ETONYEK_DEBUG_MSG(("IWORKFormula::operator()(IWORKFormula::Address): can not find the table name: %s[no correspondance tables]\n", tableName.c_str()));
+        props.insert("librevenge:sheet-name", tableName.c_str());
+      }
     }
 
     if (val.m_column)
@@ -637,11 +685,48 @@ struct Collector : public boost::static_visitor<>
     m_propsVector.append(props);
   }
 
+  void operator()(const IWORKFormula::Token &val) const
+  {
+    librevenge::RVNGPropertyList props;
+    switch (val.m_type)
+    {
+    case IWORKFormula::Token::Cell:
+      operator()(val.m_address);
+      break;
+    case IWORKFormula::Token::Double:
+      props.insert("librevenge:type", "librevenge-number");
+      props.insert("librevenge:number", val.m_value, librevenge::RVNG_GENERIC);
+      m_propsVector.append(props);
+      break;
+    case IWORKFormula::Token::Function:
+      props.insert("librevenge:type", "librevenge-function");
+      props.insert("librevenge:function", val.m_string.c_str());
+      m_propsVector.append(props);
+      break;
+    case IWORKFormula::Token::Operator:
+      props.insert("librevenge:type", "librevenge-operator");
+      props.insert("librevenge:operator", val.m_string.c_str());
+      m_propsVector.append(props);
+      break;
+    case IWORKFormula::Token::String:
+      props.insert("librevenge:type", "librevenge-text");
+      props.insert("librevenge:text", val.m_string.c_str());
+      m_propsVector.append(props);
+      break;
+#if !defined(__clang__)
+    default:
+      ETONYEK_DEBUG_MSG(("IWORKFormula::Collector::operator(): unexpected token\n"));
+      break;
+#endif
+    }
+  }
   void operator()(const recursive_wrapper<PrefixOp> &val) const
   {
     librevenge::RVNGPropertyList props;
     props.insert("librevenge:type", "librevenge-operator");
-    props.insert("librevenge:operator", val.get().m_op);
+    std::string op;
+    op+=val.get().m_op;
+    props.insert("librevenge:operator", op.c_str());
     m_propsVector.append(props);
     apply_visitor(Collector(m_propsVector, m_tableNameMap, m_offsetColumn, m_offsetRow), val.get().m_expr);
   }
@@ -721,11 +806,13 @@ private:
 
 struct IWORKFormula::Impl
 {
-  Impl() :
-    m_formula()
+  Impl()
+    : m_formula()
+    , m_tokenList()
   {
   }
   Expression m_formula;
+  std::vector<Token> m_tokenList;
 };
 
 IWORKFormula::IWORKFormula(const boost::optional<unsigned> &hc)
@@ -748,6 +835,12 @@ bool IWORKFormula::parse(const std::string &formula)
   return true;
 }
 
+bool IWORKFormula::parse(const std::vector<IWORKFormula::Token> &formula)
+{
+  m_impl->m_tokenList=formula;
+  return true;
+}
+
 const std::string IWORKFormula::str(const boost::optional<unsigned> &hc) const
 {
   ostringstream out;
@@ -755,7 +848,11 @@ const std::string IWORKFormula::str(const boost::optional<unsigned> &hc) const
   int offsetCol=0, offsetRow=0;
   if (!computeOffset(hc, offsetCol, offsetRow))
     offsetCol=offsetRow=0;
-  apply_visitor(Printer(out, offsetCol, offsetRow), m_impl->m_formula);
+  Printer printer(out, offsetCol, offsetRow);
+  if (m_impl->m_tokenList.empty())
+    apply_visitor(printer, m_impl->m_formula);
+  else
+    for (auto const &f : m_impl->m_tokenList) printer(f);
   return out.str();
 }
 
@@ -764,7 +861,11 @@ void IWORKFormula::write(const boost::optional<unsigned> &hc, librevenge::RVNGPr
   int offsetCol=0, offsetRow=0;
   if (!computeOffset(hc, offsetCol, offsetRow))
     offsetCol=offsetRow=0;
-  apply_visitor(Collector(formula, tableNameMap, offsetCol, offsetRow), m_impl->m_formula);
+  Collector collect(formula, tableNameMap, offsetCol, offsetRow);
+  if (m_impl->m_tokenList.empty())
+    apply_visitor(collect, m_impl->m_formula);
+  else
+    for (auto const &f : m_impl->m_tokenList) collect(f);
 }
 
 bool IWORKFormula::computeOffset(const boost::optional<unsigned> &hc, int &offsetColumn, int &offsetRow) const
