@@ -10,27 +10,29 @@
 #ifndef IWORKTABULARINFOELEMENT_H_INCLUDED
 #define IWORKTABULARINFOELEMENT_H_INCLUDED
 
-#include <boost/shared_ptr.hpp>
+#include <boost/optional.hpp>
+#include "IWORKStyle_fwd.h"
+#include "IWORKTypes.h"
 
 #include "IWORKXMLContextBase.h"
 
 namespace libetonyek
 {
-
 class IWORKTabularInfoElement : public IWORKXMLElementContextBase
 {
-  struct Impl;
-
 public:
   explicit IWORKTabularInfoElement(IWORKXMLParserState &state);
 
 private:
-  virtual void startOfElement();
-  virtual IWORKXMLContextPtr_t element(int name);
-  virtual void endOfElement();
+  void attribute(int name, const char *value) override;
+  void startOfElement() override;
+  IWORKXMLContextPtr_t element(int name) override;
+  void endOfElement() override;
 
-private:
-  boost::shared_ptr<Impl> m_impl;
+  boost::optional<ID_t> m_tableRef;
+  IWORKStylePtr_t m_style;
+  boost::optional<int> m_order;
+  boost::optional<IWORKWrap> m_wrap;
 };
 
 }
