@@ -10,25 +10,33 @@
 #ifndef LIBETONYEK_XML_H_INCLUDED
 #define LIBETONYEK_XML_H_INCLUDED
 
+#include <memory>
 #include <string>
 
+#include <boost/optional.hpp>
+
+#include <libxml/xmlreader.h>
+
 #include "libetonyek_utils.h"
 
-extern "C" int readFromStream(void *context, char *buffer, int len);
-
-extern "C" int closeStream(void * /* context */);
-
 namespace libetonyek
 {
 
+std::unique_ptr<xmlTextReader, void (*)(xmlTextReaderPtr)> xmlReaderForStream(const RVNGInputStreamPtr_t &input);
+
 /** Convert string value to bool.
   *
   * @arg value the string
   * @returns the boolean value of the string
   */
 bool bool_cast(const char *value);
+boost::optional<bool> try_bool_cast(const char *value);
+
 double double_cast(const char *value);
+boost::optional<double> try_double_cast(const char *value);
+
 int int_cast(const char *value);
+boost::optional<int> try_int_cast(const char *value);
 
 const char *char_cast(const char *c);
 const char *char_cast(const signed char *c);
